* {{ 
  font-family: 'Lato' !important; 
  selection-background-color: #3390ff;
  selection-color: #fff;
}}
QWidget {{
  color: #000;
  margin: 1px;
}}

QListView, 
QTreeView {{
  color: #000;
  background-color: #fff;
  border: 1px solid #c0c0c0;
}}
QListView::item:selected, 
QTreeView::item:selected {{
  color: #fff;
  selection-color: #000000;
}}
QListView::item:selected:focus, 
QTreeView::item:selected:focus {{
  color: #fff;
  selection-color: #000;
}}
QTabBar {{
  text-transform: none;
}}
QTabBar::tab:selected {{
  color: #000 !important;
}}
QTabBar::tab:top:selected,
QTabBar::tab:top:hover {{
  color: #000;
}}
QPushButton {{
  color: #3e3e3e;
  text-transform: none;
  background-color: {QTMATERIAL_SECONDARYCOLOR};
  border: 1px solid #fff;
}}
QLineEdit {{
  color: #000000; 
  border: 1px solid #c0c0c0;
  background-color: #fff;
}}
QLineEdit::focus {{
  color: #000000; 
  border: 1px solid #c0c0c0;
  background-color: #fff;
}}
QTextEdit {{
  color: #000000; 
  border: 1px solid #c0c0c0;
  background-color: #fff;
}}
QMenu {{ 
  background-color: #fff; 
  border: none;
}} 
QMenu::item {{ 
  color: #0e0e0e;
}} 
QComboBox,
QComboBox::focus {{
  background-color: #fff;
  color: #000000;
}}

QMainWindow::separator:horizontal,
QSplitter::handle:vertical {{
  width: 100%;
  height: 1px;
  background-color: #d4d4d4;
}}
QTableView::item,
QTreeView::item,
QListView::item {{
  color: #0e0e0e;
  padding: 4px;
  min-height: 20px;
}}
QCalendarWidget QWidget {{
  background: #fff;
}}
QScrollBar::handle {{
  background: {QTMATERIAL_PRIMARYCOLOR};
}}
QTreeView::item:disabled,
QListView::item:disabled,
QComboBox::item:disabled {{
  color: #828282;
  font-weight: bold
}}
QComboBox::item {{
  color: #000;
}}
QCheckBox::indicator,
QRadioButton::indicator {{
    border: 1px solid #828282 !important;
    width: 16px;
    height: 16px;
}}
QRadioButton::indicator,
QRadioButton::indicator::focus {{
  border-radius: 8px;
}}

QCheckBox::indicator:checked,
QGroupBox::indicator:checked {{
  image: none;
  background-color: #c1c2c2;
  selection-background-color: #fff; 
}}

QCheckBox::indicator:unchecked,
QGroupBox::indicator:unchecked {{
  image: none;
  background-color: #fff;
  border: 1px solid #828282 !important;
}}

QRadioButton::indicator:checked {{
  image: none;
  background-color: #c1c2c2;
  selection-background-color: #fff; 
  border-radius: 8px;
}}
QRadioButton::indicator:unchecked {{
  image: none;
  border: 1px solid #828282 !important;
  background-color: #fff;
}}
QHeaderView::section {{
  text-transform: none;
}}

QMenu::indicator {{
  border: none !important;
  width: 16px;
  height: 16px;
  image: none;
}}
QMenu::indicator:non-exclusive:unchecked {{
  image: none;
  background-color: #fff;
}}
QMenu::indicator:non-exclusive:unchecked:selected {{
  image: none;
  background-color: #fff;
}}
QMenu::indicator:non-exclusive:checked {{
  image: none;
  background-color: #c1c2c2;
}}
QMenu::indicator:non-exclusive:checked:selected {{
  image: none;
  background-color: #c1c2c2;
}}
QMenu::indicator:exclusive:unchecked {{
  image: none;
  background-color: #fff;
}}
QMenu::indicator:exclusive:unchecked:selected {{
  image: none;
  background-color: #fff;
}}
QMenu::indicator:exclusive:checked {{
  image: none;
  background-color: #c1c2c2;
}}
QMenu::indicator:exclusive:checked:selected {{
  image: none;
  background-color: #c1c2c2;
}}
QPushButton {{
  text-transform: none;
}}

.label-title {{
  font-weight: bold;
}}
.label-help {{
  color: #5d5d5d;
}}
.label-chat-status {{
  color: #5d5d5d;
}}
.interpreter-input {{
  font-family: 'Monaspace Neon';
}}
.interpreter-output,
.code-editor {{
  font-family: 'Monaspace Neon';
  background: #fff; 
  color: #000;
}}

/* radius */
QPushButton,
QTextEdit,
QListView,
QTreeView,
QComboBox,
QLineEdit,
QWebView {{
  border-radius: 10px
}}
QMenu,
.layout-input {{
  border-top-right-radius: 10px;
  border-top-left-radius: 0px;
  border-bottom-right-radius: 10px;
  border-bottom-left-radius: 10px;
}}
.layout-search {{
  height: 35px;
}}
.layout-output-web {{
  border-radius: 10px;
}}
QListView,
QTreeView,
QTextEdit,
QLineEdit {{
 border: none;
}}