* {{ 
  font-family: 'Lato' !important; 
  selection-background-color: #0765d4;
  selection-color: #fff;
}}
QLineEdit {{
  color: #ffffff; 
  border: 1px solid #5c5c5c;
}}
QTextEdit {{
  color: #ffffff; 
  border: 1px solid #5c5c5c;
}}
QTabBar {{
  text-transform: none;
}}
QComboBox {{
  color: #fff;
}}
QTreeView,
QListView,
QLineEdit,
QComboBox {{  
  height: 30px;
}}
QTableView::item,
QTreeView::item,
QListView::item {{
  padding: 4px;
  min-height: 20px;
}}
QListView::item:selected,
QTreeView::item:selected {{
  color: #ffffff;
  selection-color: #ffffff;
}}
QComboBox::item:selected {{
  color: #000;
  selection-color: #000;
}}
QComboBox::item:disabled {{
  color: #d1d2d2;
}}
QListView::item:selected:focus, 
QComboBox::item:selected:focus,
QTreeView::item:selected:focus {{
  color: #ffffff;
  selection-color: #000000;
}}
QListView::item::disabled {{
   color: #959595;
   font-weight: bold;
}}
QMenuBar::item:selected {{ 
  color: #000000;
}} 
QMenu::item:selected {{ 
  color: #000000;
}} 
QPushButton {{
  text-transform: none;
}}
QHeaderView::section {{
  text-transform: none;
}}

.label-title {{
  font-weight: bold;
}}
.label-help {{
  color: #999;
}}
.label-chat-status {{
  color: #999;
}}
.label-link {{
  color: #90d9ff;
  font-weight: bold;
  text-decoration: none;
}}
.interpreter-input {{
  font-family: 'Monaspace Neon';
}}
.interpreter-output,
.code-editor {{
  font-family: 'Monaspace Neon';
  background: #232629; 
  color: #fff;
}}

/* radius */
QPushButton,
QTextEdit,
QListView,
QTreeView,
QComboBox,
QLineEdit,
QWebView {{
  border-radius: 10px
}}
QMenu,
.layout-input,
.layout-notepad {{
  border-top-right-radius: 10px;
  border-top-left-radius: 0px;
  border-bottom-right-radius: 10px;
  border-bottom-left-radius: 10px;
}}
.layout-search {{
  height: 35px;
}}
.layout-output-web {{
  border-radius: 10px;
}}
QTextEdit,
QLineEdit {{
 border: none;
}}